@@ -17,6 +17,8 @@
 */
 
 #include "global.h"
+#include "kioglobal_p.h"
+#include "faviconscache_p.h"
 
 #include <kprotocolinfo.h>
 #include <kconfig.h>
@@ -24,10 +26,7 @@
 #include <klocalizedstring.h>
 #include <ksharedconfig.h>
 #include <qmimedatabase.h>
-#include <QDBusInterface>
-#include <QDBusReply>
 #include <QUrl>
-#include <QHash>
 #include <QLocale>
 #include <QFileInfo>
 
@@ -51,7 +50,7 @@ BinaryUnitDialect _k_loadBinaryDialect()
     KConfigGroup mainGroup(KSharedConfig::openConfig(), "Locale");
 
     BinaryUnitDialect dialect(BinaryUnitDialect(mainGroup.readEntry("BinaryUnitDialect", int(DefaultBinaryDialect))));
-    dialect = (BinaryUnitDialect) mainGroup.readEntry("BinaryUnitDialect", int(dialect));
+    dialect = static_cast<BinaryUnitDialect>(mainGroup.readEntry("BinaryUnitDialect", int(dialect)));
 
     // Error checking
     if (dialect <= DefaultBinaryDialect || dialect > LastBinaryDialect) {
@@ -165,7 +164,7 @@ KIOCORE_EXPORT QString KIO::convertSeconds(unsigned int seconds)
     seconds            = (seconds - (days * 86400) - (hours * 3600) - (mins * 60));
 
     const QTime time(hours, mins, seconds);
-    const QString timeStr(time.toString("hh:mm:ss"));
+    const QString timeStr(time.toString(QStringLiteral("hh:mm:ss")));
     if (days > 0) {
         return i18np("1 day %2", "%1 days %2", days, timeStr);
     } else {
@@ -249,19 +248,19 @@ KIO::CacheControl KIO::parseCacheControl(const QString &cacheControl)
 {
     QString tmp = cacheControl.toLower();
 
-    if (tmp == "cacheonly") {
+    if (tmp == QLatin1String("cacheonly")) {
         return KIO::CC_CacheOnly;
     }
-    if (tmp == "cache") {
+    if (tmp == QLatin1String("cache")) {
         return KIO::CC_Cache;
     }
-    if (tmp == "verify") {
+    if (tmp == QLatin1String("verify")) {
         return KIO::CC_Verify;
     }
-    if (tmp == "refresh") {
+    if (tmp == QLatin1String("refresh")) {
         return KIO::CC_Refresh;
     }
-    if (tmp == "reload") {
+    if (tmp == QLatin1String("reload")) {
         return KIO::CC_Reload;
     }
 
@@ -272,73 +271,32 @@ KIO::CacheControl KIO::parseCacheControl(const QString &cacheControl)
 QString KIO::getCacheControlString(KIO::CacheControl cacheControl)
 {
     if (cacheControl == KIO::CC_CacheOnly) {
-        return "CacheOnly";
+        return QStringLiteral("CacheOnly");
     }
     if (cacheControl == KIO::CC_Cache) {
-        return "Cache";
+        return QStringLiteral("Cache");
     }
     if (cacheControl == KIO::CC_Verify) {
-        return "Verify";
+        return QStringLiteral("Verify");
     }
     if (cacheControl == KIO::CC_Refresh) {
-        return "Refresh";
+        return QStringLiteral("Refresh");
     }
     if (cacheControl == KIO::CC_Reload) {
-        return "Reload";
+        return QStringLiteral("Reload");
     }
     qCDebug(KIO_CORE) << "unrecognized Cache control enum value:" << cacheControl;
     return QString();
 }
 
-static bool useFavIcons()
-{
-    // this method will be called quite often, so better not read the config
-    // again and again.
-    static bool s_useFavIconsChecked = false;
-    static bool s_useFavIcons = false;
-    if (!s_useFavIconsChecked) {
-        s_useFavIconsChecked = true;
-        KConfigGroup cg(KSharedConfig::openConfig(), "HTML Settings");
-        s_useFavIcons = cg.readEntry("EnableFavicon", true);
-    }
-    return s_useFavIcons;
-}
-
 QString KIO::favIconForUrl(const QUrl &url)
 {
-    /* The kded module also caches favicons, for one week, without any way
-     * to clean up the cache meanwhile.
-     * On the other hand, this QHash will get cleaned up after 5000 request
-     * (a selection in konsole of 80 chars generates around 500 requests)
-     * or by simply restarting the application (or the whole desktop,
-     * more likely, for the case of konqueror or konsole).
-     */
-    static QHash<QUrl, QString> iconNameCache;
-    static int autoClearCache = 0;
-    const QString notFound = QLatin1String("NOTFOUND");
-
     if (url.isLocalFile()
-            || !url.scheme().startsWith(QLatin1String("http"))
-            || !useFavIcons()) {
+        || !url.scheme().startsWith(QLatin1String("http"))) {
         return QString();
     }
 
-    QString iconNameFromCache = iconNameCache.value(url, notFound);
-    if (iconNameFromCache != notFound) {
-        if ((++autoClearCache) < 5000) {
-            return iconNameFromCache;
-        } else {
-            iconNameCache.clear();
-            autoClearCache = 0;
-        }
-    }
-
-    QDBusInterface kded(QString::fromLatin1("org.kde.kded5"),
-                        QString::fromLatin1("/modules/favicons"),
-                        QString::fromLatin1("org.kde.FavIcon"));
-    QDBusReply<QString> result = kded.call(QString::fromLatin1("iconForUrl"), url.toString());
-    iconNameCache.insert(url, result.value());
-    return result;              // default is QString()
+    return FavIconsCache::instance()->iconForUrl(url);
 }
 
 QString KIO::iconNameForUrl(const QUrl &url)
@@ -349,12 +307,27 @@ QString KIO::iconNameForUrl(const QUrl &url)
     const QString mimeTypeIcon = mt.iconName();
     QString i = mimeTypeIcon;
 
+    // check whether it's a xdg location (e.g. Pictures folder)
+    if (url.isLocalFile() && mt.inherits(QLatin1String("inode/directory"))) {
+        i = KIOPrivate::iconForStandardPath(url.toLocalFile());
+    }
+
     // if we don't find an icon, maybe we can use the one for the protocol
     if (i == unknown || i.isEmpty() || mt.isDefault()
             // and for the root of the protocol (e.g. trash:/) the protocol icon has priority over the mimetype icon
             || url.path().length() <= 1) {
         i = favIconForUrl(url); // maybe there is a favicon?
 
+        // reflect actual fill state of trash can
+        if (url.scheme() == QLatin1String("trash") && url.path().length() <= 1) {
+            KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
+            if (trashConfig.group("Status").readEntry("Empty", true)) {
+                i = QStringLiteral("user-trash");
+            } else {
+                i = QStringLiteral("user-trash-full");
+            }
+        }
+
         if (i.isEmpty()) {
             i = KProtocolInfo::icon(url.scheme());
         }
@@ -394,25 +367,31 @@ QString KIO::suggestName(const QUrl &baseURL, const QString &oldName)
     QString nameSuffix = db.suffixForFileName(oldName);
 
     if (oldName.lastIndexOf('.') == 0) {
-        basename = ".";
+        basename = QStringLiteral(".");
         nameSuffix = oldName;
     } else if (nameSuffix.isEmpty()) {
-        basename = oldName;
+        const int lastDot = oldName.lastIndexOf('.');
+        if (lastDot == -1) {
+            basename = oldName;
+        } else {
+            basename = oldName.left(lastDot);
+            nameSuffix = oldName.mid(lastDot);
+        }
     } else {
         nameSuffix.prepend('.');
         basename = oldName.left(oldName.length() - nameSuffix.length());
     }
 
     // check if (number) exists from the end of the oldName and increment that number
-    QRegExp numSearch("\\(\\d+\\)");
+    QRegExp numSearch(QStringLiteral("\\(\\d+\\)"));
     int start = numSearch.lastIndexIn(oldName);
     if (start != -1) {
         QString numAsStr = numSearch.cap(0);
-        QString number = QString::number(numAsStr.mid(1, numAsStr.size() - 2).toInt() + 1);
+        QString number = QString::number(numAsStr.midRef(1, numAsStr.size() - 2).toInt() + 1);
         basename = basename.left(start) + '(' + number + ')';
     } else {
         // number does not exist, so just append " (1)" to filename
-        basename += " (1)";
+        basename += QLatin1String(" (1)");
     }
     const QString suggestedName = basename + nameSuffix;
 
