@@ -43,6 +43,8 @@
 #include <QtCore/QTextCodec>
 #include <QtCore/QCryptographicHash>
 
+Q_LOGGING_CATEGORY(KIO_HTTP_AUTH, "kf5.kio.kio_http.auth")
+
 static bool isWhiteSpace(char ch)
 {
     return (ch == ' ' || ch == '\t' || ch == '\v' || ch == '\f');
@@ -73,7 +75,7 @@ static bool containsScheme(const char input[], int start, int end)
 // keys on even indexes, values on odd indexes. Reduces code expansion for the templated
 // alternatives.
 // If "ba" starts with empty content it will be removed from ba to simplify later calls
-static QList<QByteArray> parseChallenge(QByteArray &ba, QByteArray *scheme, QByteArray *nextAuth = 0)
+static QList<QByteArray> parseChallenge(QByteArray &ba, QByteArray *scheme, QByteArray *nextAuth = nullptr)
 {
     QList<QByteArray> values;
     const char *b = ba.constData();
@@ -273,7 +275,7 @@ KAbstractHttpAuthentication *KAbstractHttpAuthentication::newAuth(const QByteArr
         } else if (scheme == "basic") { // krazy:exclude=strings
             return new KHttpBasicAuthentication();
         }
-    return 0;
+    return nullptr;
 }
 
 QList< QByteArray > KAbstractHttpAuthentication::splitOffers(const QList< QByteArray > &offers)
@@ -322,14 +324,14 @@ void KAbstractHttpAuthentication::setChallenge(const QByteArray &c, const QUrl &
     m_challenge = parseChallenge(m_challengeText, &m_scheme);
     Q_ASSERT(m_scheme.toLower() == scheme().toLower());
     m_resource = resource;
-    m_httpMethod = httpMethod.trimmed();
+    m_httpMethod = httpMethod;
 }
 
 QString KAbstractHttpAuthentication::realm() const
 {
     const QByteArray realm = valueForKey(m_challenge, "realm");
     // TODO: Find out what this is supposed to address. The site mentioned below does not exist.
-    if (QLocale().uiLanguages().contains(QLatin1String("ru"))) {
+    if (QLocale().uiLanguages().contains(QStringLiteral("ru"))) {
         //for sites like lib.homelinux.org
         return QTextCodec::codecForName("CP1251")->toUnicode(realm);
     }
@@ -564,11 +566,11 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
         if (info.qop.contains("auth")) {
             info.qop = "auth";
         } else {
-            qWarning() << "Unsupported digest authentication qop parameters:" << values;
+            qCWarning(KIO_HTTP_AUTH) << "Unsupported digest authentication qop parameters:" << values;
             info.qop.clear();
         }
     } else if (info.qop == "auth-int") {
-        qWarning() << "Unsupported digest authentication qop parameter:" << info.qop;
+        qCWarning(KIO_HTTP_AUTH) << "Unsupported digest authentication qop parameter:" << info.qop;
         info.qop.clear();
     }
 
@@ -700,7 +702,7 @@ void KHttpNtlmAuthentication::setChallenge(const QByteArray &c, const QUrl &reso
     }
     // The type 1 message we're going to send needs no credentials;
     // they come later in the type 3 message.
-    m_needCredentials = m_challenge.isEmpty();
+    m_needCredentials = !m_challenge.isEmpty();
 }
 
 void KHttpNtlmAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
@@ -709,7 +711,7 @@ void KHttpNtlmAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
     // Every auth scheme is supposed to supply a realm according to the RFCs. Of course this doesn't
     // prevent Microsoft from not doing it... Dummy value!
     // we don't have the username yet which may (may!) contain a domain, so we really have no choice
-    ai->realmValue = QLatin1String("NTLM");
+    ai->realmValue = QStringLiteral("NTLM");
 }
 
 void KHttpNtlmAuthentication::generateResponse(const QString &_user, const QString &password)
@@ -724,8 +726,27 @@ void KHttpNtlmAuthentication::generateResponse(const QString &_user, const QStri
     if (m_challenge.isEmpty()) {
         m_finalAuthStage = false;
         // first, send type 1 message (with empty domain, workstation..., but it still works)
-        if (!KNTLM::getNegotiate(buf)) {
-            qWarning() << "Error while constructing Type 1 NTLM authentication request";
+        switch (m_stage1State) {
+        case Init:
+            if (!KNTLM::getNegotiate(buf)) {
+                qCWarning(KIO_HTTP_AUTH) << "Error while constructing Type 1 NTLMv1 authentication request";
+                m_isError = true;
+                return;
+            }
+            m_stage1State = SentNTLMv1;
+            break;
+        case SentNTLMv1:
+            if (!KNTLM::getNegotiate(buf, QString(), QString(), KNTLM::Negotiate_NTLM2_Key
+                                     | KNTLM::Negotiate_Always_Sign | KNTLM::Negotiate_Unicode
+                                     | KNTLM::Request_Target | KNTLM::Negotiate_NTLM)) {
+                qCWarning(KIO_HTTP_AUTH) << "Error while constructing Type 1 NTLMv2 authentication request";
+                m_isError = true;
+                return;
+            }
+            m_stage1State = SentNTLMv2;
+            break;
+        default:
+            qCWarning(KIO_HTTP_AUTH) << "Error - Type 1 NTLM already sent - no Type 2 response received.";
             m_isError = true;
             return;
         }
@@ -744,12 +765,12 @@ void KHttpNtlmAuthentication::generateResponse(const QString &_user, const QStri
         const QByteArray challenge = QByteArray::fromBase64(m_challenge[0]);
 
         KNTLM::AuthFlags flags = KNTLM::Add_LM;
-        if (!m_config || !m_config->readEntry("EnableNTLMv2Auth", false)) {
+        if ((!m_config || !m_config->readEntry("EnableNTLMv2Auth", false)) && (m_stage1State != SentNTLMv2)) {
             flags |= KNTLM::Force_V1;
         }
 
-        if (!KNTLM::getAuth(buf, challenge, user, m_password, domain, QLatin1String("WORKSTATION"), flags)) {
-            qWarning() << "Error while constructing Type 3 NTLM authentication request";
+        if (!KNTLM::getAuth(buf, challenge, user, m_password, domain, QStringLiteral("WORKSTATION"), flags)) {
+            qCWarning(KIO_HTTP_AUTH) << "Error while constructing Type 3 NTLM authentication request";
             m_isError = true;
             return;
         }
@@ -831,7 +852,7 @@ void KHttpNegotiateAuthentication::generateResponse(const QString &user, const Q
     // see whether we can use the SPNEGO mechanism
     major_status = gss_indicate_mechs(&minor_status, &mech_set);
     if (GSS_ERROR(major_status)) {
-        qDebug() << "gss_indicate_mechs failed:" << gssError(major_status, minor_status);
+        qCDebug(KIO_HTTP_AUTH) << "gss_indicate_mechs failed:" << gssError(major_status, minor_status);
     } else {
         for (uint i = 0; i < mech_set->count; i++) {
             tmp_oid = &mech_set->elements[i];
@@ -855,11 +876,11 @@ void KHttpNegotiateAuthentication::generateResponse(const QString &user, const Q
     major_status = gss_import_name(&minor_status, &input_token,
                                    GSS_C_NT_HOSTBASED_SERVICE, &server);
 
-    input_token.value = NULL;
+    input_token.value = nullptr;
     input_token.length = 0;
 
     if (GSS_ERROR(major_status)) {
-        qDebug() << "gss_import_name failed:" << gssError(major_status, minor_status);
+        qCDebug(KIO_HTTP_AUTH) << "gss_import_name failed:" << gssError(major_status, minor_status);
         m_isError = true;
         return;
     }
@@ -876,11 +897,11 @@ void KHttpNegotiateAuthentication::generateResponse(const QString &user, const Q
                                         &ctx, server, mech_oid,
                                         req_flags, GSS_C_INDEFINITE,
                                         GSS_C_NO_CHANNEL_BINDINGS,
-                                        GSS_C_NO_BUFFER, NULL, &output_token,
-                                        NULL, NULL);
+                                        GSS_C_NO_BUFFER, nullptr, &output_token,
+                                        nullptr, nullptr);
 
     if (GSS_ERROR(major_status) || (output_token.length == 0)) {
-        qDebug() << "gss_init_sec_context failed:" << gssError(major_status, minor_status);
+        qCDebug(KIO_HTTP_AUTH) << "gss_init_sec_context failed:" << gssError(major_status, minor_status);
         gss_release_name(&minor_status, &server);
         if (ctx != GSS_C_NO_CONTEXT) {
             gss_delete_sec_context(&minor_status, &ctx, GSS_C_NO_BUFFER);
