@@ -21,6 +21,8 @@
 
 #include "kio_trash.h"
 #include "testtrash.h"
+#include "../../../pathhelpers_p.h"
+
 #include <kprotocolinfo.h>
 #include <QTemporaryFile>
 #include <QDataStream>
@@ -83,7 +85,11 @@ QString TestTrash::readOnlyDirPath() const
 QString TestTrash::otherTmpDir() const
 {
     // This one needs to be on another partition for the test to be meaningful
-    return m_tempDir.path() + QLatin1Char('/');
+    QString tempDir = m_tempDir.path();
+    if (!tempDir.endsWith('/')) {
+        tempDir.append('/');
+    }
+    return tempDir;
 }
 
 QString TestTrash::utf8FileName() const
@@ -636,21 +642,6 @@ void TestTrash::trashDirectoryWithTrailingSlash()
     trashDirectory(homeTmpDir() + dirName, QStringLiteral("dirwithslash"));
 }
 
-void TestTrash::tryRenameInsideTrash()
-{
-    qDebug() << " with file_move";
-    KIO::Job *job = KIO::file_move(QUrl(QStringLiteral("trash:/0-tryRenameInsideTrash")), QUrl(QStringLiteral("trash:/foobar")), -1, KIO::HideProgressInfo);
-    bool worked = job->exec();
-    QVERIFY(!worked);
-    QVERIFY(job->error() == KIO::ERR_CANNOT_RENAME);
-
-    qDebug() << " with move";
-    job = KIO::move(QUrl(QStringLiteral("trash:/0-tryRenameInsideTrash")), QUrl(QStringLiteral("trash:/foobar")), KIO::HideProgressInfo);
-    worked = job->exec();
-    QVERIFY(!worked);
-    QVERIFY(job->error() == KIO::ERR_CANNOT_RENAME);
-}
-
 void TestTrash::delRootFile()
 {
     // test deleting a trashed file
@@ -798,7 +789,7 @@ void TestTrash::copyFromTrash(const QString &fileId, const QString &destPath, co
 {
     QUrl src(QLatin1String("trash:/0-") + fileId);
     if (!relativePath.isEmpty()) {
-        src.setPath(src.path() + '/' + relativePath);
+        src.setPath(concatPaths(src.path(), relativePath));
     }
     QUrl dest = QUrl::fromLocalFile(destPath);
 
@@ -855,11 +846,61 @@ void TestTrash::copySymlinkFromTrash() // relies on trashSymlinkFromHome() being
     QVERIFY(QFileInfo(destPath).isSymLink());
 }
 
+void TestTrash::moveInTrash(const QString &fileId, const QString &destFileId)
+{
+    const QUrl src(QLatin1String("trash:/0-") + fileId);
+    const QUrl dest(QLatin1String("trash:/") + destFileId);
+
+    QVERIFY(MyNetAccess_exists(src));
+    QVERIFY(!MyNetAccess_exists(dest));
+
+    KIO::Job *job = KIO::moveAs(src, dest, KIO::HideProgressInfo);
+    bool ok = job->exec();
+    QVERIFY2(ok, qPrintable(job->errorString()));
+
+    // Check old doesn't exist anymore
+    QString infoFile(m_trashDir + "/info/" + fileId + ".trashinfo");
+    QVERIFY(!QFile::exists(infoFile));
+    QFileInfo filesItem(m_trashDir + "/files/" + fileId);
+    QVERIFY(!filesItem.exists());
+
+    // Check new exists now
+    QString newInfoFile(m_trashDir + "/info/" + destFileId + ".trashinfo");
+    QVERIFY(QFile::exists(newInfoFile));
+    QFileInfo newFilesItem(m_trashDir + "/files/" + destFileId);
+    QVERIFY(newFilesItem.exists());
+
+}
+
+void TestTrash::renameFileInTrash()
+{
+    const QString fileName = QStringLiteral("renameFileInTrash");
+    const QString filePath = homeTmpDir() + fileName;
+    createTestFile(filePath);
+    trashFile(filePath, fileName);
+
+    const QString destFileName = QStringLiteral("fileRenamed");
+    moveInTrash(fileName, destFileName);
+
+    // cleanup
+    KIO::Job *delJob = KIO::del(QUrl(QStringLiteral("trash:/0-fileRenamed")), KIO::HideProgressInfo);
+    bool ok = delJob->exec();
+    QVERIFY2(ok, qPrintable(delJob->errorString()));
+}
+
+void TestTrash::renameDirInTrash()
+{
+    const QString dirName = QStringLiteral("trashDirFromHome");
+    const QString destDirName = QStringLiteral("dirRenamed");
+    moveInTrash(dirName, destDirName);
+    moveInTrash(destDirName, dirName);
+}
+
 void TestTrash::moveFromTrash(const QString &fileId, const QString &destPath, const QString &relativePath)
 {
     QUrl src(QLatin1String("trash:/0-") + fileId);
     if (!relativePath.isEmpty()) {
-        src.setPath(src.path() + '/' + relativePath);
+        src.setPath(concatPaths(src.path(), relativePath));
     }
     QUrl dest = QUrl::fromLocalFile(destPath);
 
@@ -1227,25 +1268,43 @@ void TestTrash::emptyTrash()
 #endif
 }
 
+static bool isTrashEmpty()
+{
+    KConfig cfg(QStringLiteral("trashrc"), KConfig::SimpleConfig);
+    const KConfigGroup group = cfg.group("Status");
+    return group.readEntry("Empty", true);
+}
+
 void TestTrash::testEmptyTrashSize()
 {
     KIO::DirectorySizeJob *job = KIO::directorySize(QUrl(QStringLiteral("trash:/")));
     QVERIFY(job->exec());
-    QVERIFY(job->totalSize() < 1000000000 /*1GB*/); // #157023
+    if (isTrashEmpty()) {
+        QCOMPARE(job->totalSize(), 0ULL);
+    } else {
+        QVERIFY(job->totalSize() < 1000000000 /*1GB*/); // #157023
+    }
 }
 
 static void checkIcon(const QUrl &url, const QString &expectedIcon)
 {
-    QString icon = KIO::iconNameForUrl(url);
+    QString icon = KIO::iconNameForUrl(url); // #100321
     QCOMPARE(icon, expectedIcon);
 }
 
 void TestTrash::testIcons()
 {
+    // The JSON file says "user-trash-full" in all cases, whether the trash is full or not
     QCOMPARE(KProtocolInfo::icon(QStringLiteral("trash")), QStringLiteral("user-trash-full"));  // #100321
-    checkIcon(QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash-full"));   // #100321
+
+    if (isTrashEmpty()) {
+        checkIcon(QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"));
+    } else {
+        checkIcon(QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash-full"));
+    }
+
     checkIcon(QUrl(QStringLiteral("trash:/foo/")), QStringLiteral("inode-directory"));
 }
 
-QTEST_MAIN(TestTrash) // QT5 TODO: NOGUI
+QTEST_GUILESS_MAIN(TestTrash)
 
